@@ -3,18 +3,17 @@
  * joinpath.c
  *	  Routines to find all possible paths for processing a set of joins
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/optimizer/path/joinpath.c,v 1.67.2.1 2005/01/23 02:25:25 tgl Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/optimizer/path/joinpath.c,v 1.71.2.2 2005/10/25 20:30:52 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
 #include "postgres.h"
 
-#include <sys/types.h>
 #include <math.h>
 
 #include "optimizer/clauses.h"
@@ -24,6 +23,7 @@
 #include "parser/parsetree.h"
 #include "utils/lsyscache.h"
 
+
 static void sort_inner_and_outer(Query *root, RelOptInfo *joinrel,
 					 RelOptInfo *outerrel, RelOptInfo *innerrel,
 					 List *restrictlist, List *mergeclause_list,
@@ -171,7 +171,7 @@ sort_inner_and_outer(Query *root,
 		default:
 			elog(ERROR, "sort_inner_and_outer: unexpected join type %d",
 				 (int) jointype);
-			useallclauses = false;	/* keep compiler quiet */
+			useallclauses = false;		/* keep compiler quiet */
 			break;
 	}
 
@@ -721,8 +721,6 @@ hash_inner_and_outer(Query *root,
 					 List *restrictlist,
 					 JoinType jointype)
 {
-	Relids		outerrelids = outerrel->relids;
-	Relids		innerrelids = innerrel->relids;
 	bool		isouterjoin;
 	List	   *i;
 
@@ -774,13 +772,13 @@ hash_inner_and_outer(Query *root,
 		/*
 		 * Check if clause is usable with these input rels.
 		 */
-		if (intMember(left->varno, outerrelids) &&
-			intMember(right->varno, innerrelids))
+		if (VARISRELMEMBER(left->varno, outerrel) &&
+			VARISRELMEMBER(right->varno, innerrel))
 		{
 			/* righthand side is inner */
 		}
-		else if (intMember(left->varno, innerrelids) &&
-				 intMember(right->varno, outerrelids))
+		else if (VARISRELMEMBER(left->varno, innerrel) &&
+				 VARISRELMEMBER(right->varno, outerrel))
 		{
 			/* lefthand side is inner */
 		}
@@ -902,9 +900,8 @@ select_mergejoin_clauses(RelOptInfo *joinrel,
 						 JoinType jointype)
 {
 	List	   *result_list = NIL;
-	Relids		outerrelids = outerrel->relids;
-	Relids		innerrelids = innerrel->relids;
 	bool		isouterjoin = IS_OUTER_JOIN(jointype);
+	bool		have_nonmergeable_joinclause = false;
 	List	   *i;
 
 	foreach(i, restrictlist)
@@ -917,47 +914,50 @@ select_mergejoin_clauses(RelOptInfo *joinrel,
 		/*
 		 * If processing an outer join, only use its own join clauses in
 		 * the merge.  For inner joins we need not be so picky.
-		 *
-		 * Furthermore, if it is a right/full join then *all* the explicit
-		 * join clauses must be mergejoinable, else the executor will
-		 * fail. If we are asked for a right join then just return NIL to
-		 * indicate no mergejoin is possible (we can handle it as a left
-		 * join instead). If we are asked for a full join then emit an
-		 * error, because there is no fallback.
 		 */
-		if (isouterjoin)
-		{
-			if (restrictinfo->ispusheddown)
-				continue;
-			switch (jointype)
-			{
-				case JOIN_RIGHT:
-					if (restrictinfo->mergejoinoperator == InvalidOid)
-						return NIL;		/* not mergejoinable */
-					break;
-				case JOIN_FULL:
-					if (restrictinfo->mergejoinoperator == InvalidOid)
-						elog(ERROR, "FULL JOIN is only supported with mergejoinable join conditions");
-					break;
-				default:
-					/* otherwise, it's OK to have nonmergeable join quals */
-					break;
-			}
-		}
+		if (isouterjoin && restrictinfo->ispusheddown)
+			continue;
 
 		if (restrictinfo->mergejoinoperator == InvalidOid)
+		{
+			have_nonmergeable_joinclause = true;
 			continue;			/* not mergejoinable */
+		}
 
 		clause = restrictinfo->clause;
 		/* these must be OK, since check_mergejoinable accepted the clause */
 		left = get_leftop(clause);
 		right = get_rightop(clause);
 
-		if ((intMember(left->varno, outerrelids) &&
-			 intMember(right->varno, innerrelids)) ||
-			(intMember(left->varno, innerrelids) &&
-			 intMember(right->varno, outerrelids)))
+		if ((VARISRELMEMBER(left->varno, outerrel) &&
+			 VARISRELMEMBER(right->varno, innerrel)) ||
+			(VARISRELMEMBER(left->varno, innerrel) &&
+			 VARISRELMEMBER(right->varno, outerrel)))
 			result_list = lcons(restrictinfo, result_list);
+		else
+			have_nonmergeable_joinclause = true;
+	}
+
+	/*
+	 * If it is a right/full join then *all* the explicit join clauses must be
+	 * mergejoinable, else the executor will fail. If we are asked for a right
+	 * join then just return NIL to indicate no mergejoin is possible (we can
+	 * handle it as a left join instead). If we are asked for a full join then
+	 * emit an error, because there is no fallback.
+	 */
+	if (have_nonmergeable_joinclause)
+	{
+		switch (jointype)
+		{
+			case JOIN_RIGHT:
+				return NIL;		/* not mergejoinable */
+			case JOIN_FULL:
+				elog(ERROR, "FULL JOIN is only supported with mergejoinable join conditions");
+				break;
+			default:
+				/* otherwise, it's OK to have nonmergeable join quals */
+				break;
+		}
 	}
 
 	return result_list;
